/*
    tstore - an archive format for archiving tar archives
    Copyright (C) 2004  Kasper Dupont

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    with permission from the author any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "tstore.h"

uint8_t pointers[8][512];

int main(int argc, char ** argv)
{
  uint64_t store_size;
  uint8_t buf[512];
  uint64_t idx,nidx;
  uint64_t i;
  int store_fd;

  if (argc != 3) {
    fprintf(stderr,"Uage: %s <store> <file>\n",argv[0]);
    return 1;
  }

  store_fd=open(argv[1],O_RDONLY);
  if (store_fd==-1) {
    perror(argv[1]);
    return 1;
  }

  store_size=test_store_size(store_fd,1);
  store_error(store_size,argv[1]);

  assert(pread(store_fd,buf,5,13)==5);

  for (idx=GET_BNUM(buf); idx; idx=nidx) {
    get_block(idx,buf,argv[1],store_fd);
    if (!strcmp(argv[2],buf+17)) {
      uint64_t size=GET_SIZE(buf+10);
      uint64_t directory=GET_BNUM(buf+5);
      int levels=filelevels(size);
      //fprintf(stderr,">>>%d\n",levels);

      for (i=0;i<(size+511)/512;++i) {
	int j;
	int siz;
	uint64_t block=directory;
	//fprintf(stderr,"%d\n",i);
	//fprintf(stderr,"%d\n",i);
	for (j=levels-1;j>=0;--j) {
	  int idx,k;
	  idx=i;
	  for (k=0;k<j;++k) idx/=102;
	  get_block(block,(void*)pointers[j],argv[1],store_fd);
	  //fprintf(stderr,"  %d\n",(idx%102));
	  block=GET_BNUM(pointers[j]+(idx%102)*5);
	  //printf("%d\n",block);
	}
	get_block(block,buf,argv[1],store_fd);
	siz=512;
	if (i==size/512) siz=size%512;
	assert(write(1,buf,siz)==siz);
      }
      return 0;
    }
    nidx=GET_BNUM(buf);
    if ((nidx>=store_size-1)||(nidx==idx)) {
      fprintf(stderr,"Invalid link %lld=>%lld\n",idx,nidx);
      return 1;
    }
  }
  fprintf(stderr,"%s: not found\n",argv[2]);
  return 1;
}
